@@ -21,8 +21,10 @@
 
 """Formats docstrings to follow PEP 257."""
 
-from __future__ import print_function
-from __future__ import unicode_literals
+from __future__ import (absolute_import,
+                        division,
+                        print_function,
+                        unicode_literals)
 
 import io
 import os
@@ -31,7 +33,7 @@ import textwrap
 import tokenize
 
 
-__version__ = '0.5.1'
+__version__ = '0.5.2'
 
 
 try:
@@ -208,7 +210,7 @@ def format_docstring(indentation, docstring,
 
 
 def reindent(text, indentation):
-    """Reindent text to match indentation."""
+    """Return reindented text that matches indentation."""
     if '\t' not in indentation:
         text = text.expandtabs(len(indentation))
 
@@ -222,8 +224,9 @@ def reindent(text, indentation):
 def is_probably_beginning_of_sentence(line):
     """Return True if this line begins a new sentence."""
     # Check heuristically for a parameter list.
-    if re.search(r'\s-\s', line):
-        return True
+    for token in ['@', '-', r'\*']:
+        if re.search(r'\s' + token + r'\s', line):
+            return True
 
     character = line.strip()[0]
     return not character.isalnum()
@@ -235,38 +238,107 @@ def split_summary_and_description(contents):
     Return tuple (summary, description).
 
     """
-    split = contents.splitlines()
-    if len(split) > 1 and not split[1].strip():
-        # Empty line separation would indicate the rest is the description.
-        return (split[0], '\n'.join(split[2:]))
-    elif len(split) > 1 and is_probably_beginning_of_sentence(split[1]):
-        # Symbol on second line probably is a description with a list.
-        return (split[0], '\n'.join(split[1:]))
+    split_lines = contents.rstrip().splitlines()
 
-    if len(split) > 1:
-        for index, line in enumerate(split):
-            if not line.strip():
-                return ('\n'.join(split[:index]).strip(),
-                        '\n'.join(split[index:]).rstrip())
+    for index in range(1, len(split_lines)):
+        found = False
+
+        if not split_lines[index].strip():
+            # Empty line separation would indicate the rest is the description.
+            found = True
+        elif is_probably_beginning_of_sentence(split_lines[index]):
+            # Symbol on second line probably is a description with a list.
+            found = True
+
+        if found:
+            return ('\n'.join(split_lines[:index]).strip(),
+                    '\n'.join(split_lines[index:]).rstrip())
 
     # Break on first sentence.
-    for punctuation in '.?!:':
-        split = re.split('\\' + punctuation + r'\s',
-                         string=contents,
-                         maxsplit=1)
-        if len(split) == 2:
-            return (split[0].strip() + punctuation,
-                    split[1].rstrip())
+    split = split_first_sentence(contents)
+    if split[0].strip() and split[1].strip():
+        return (
+            split[0].strip(),
+            _find_shortest_indentation(
+                split[1].splitlines()[1:]) + split[1].strip()
+        )
 
-    # Break on first bullet-list-like text.
-    for token in ['    @',
-                  '    -',
-                  '    *']:
-        split = contents.split(token, 1)
-        if len(split) == 2:
-            return (split[0].strip(), (token + split[1]).strip())
+    if is_some_sort_of_list(contents):
+        # This is probably a long list of items. We should consider it a
+        # description rather than a summary to avoid screwing up the
+        # formatting.
+        return ('', contents)
+    else:
+        return (contents, '')
 
-    return (split[0], '\n'.join(split[1:]))
+
+def split_first_sentence(text):
+    """Split text into first sentence and the rest."""
+    sentence = ''
+    rest = text
+    delimiter = ''
+    previous_delimiter = ''
+
+    while rest:
+        split = re.split(r'(\s)', rest, maxsplit=1)
+        if len(split) == 3:
+            word = split[0]
+            delimiter = split[1]
+            rest = split[2]
+        elif len(split) == 1:
+            word = split[0]
+            delimiter = ''
+            rest = ''
+        else:
+            assert not split
+            break
+
+        if word in '\t':
+            continue
+
+        sentence += previous_delimiter + word
+
+        if word.endswith(('e.g.', 'i.e.',
+                          'Dr.',
+                          'Mr.', 'Mrs.', 'Ms.')):
+            # Ignore false end of sentence.
+            pass
+        elif word.endswith(('.', '?', '!')):
+            break
+        elif word.endswith(':') and delimiter == '\n':
+            # Break on colon if it ends the line. This is a heuristic to detect
+            # the beginning of some parameter list afterwards.
+            break
+
+        previous_delimiter = delimiter
+
+    return (sentence, delimiter + rest)
+
+
+def is_some_sort_of_list(text):
+    """Return True if text looks like a list."""
+    split_lines = text.rstrip().splitlines()
+    if len(split_lines) > max([len(line.strip()) for line in split_lines] +
+                              [0]):
+        return True
+
+    return False
+
+
+def _find_shortest_indentation(lines):
+    """Return most shortest indentation."""
+    assert not isinstance(lines, str)
+
+    indentation = None
+
+    for line in lines:
+        if line.strip():
+            non_whitespace_index = len(line) - len(line.lstrip())
+            _indent = line[:non_whitespace_index]
+            if indentation is None or len(_indent) < len(indentation):
+                indentation = _indent
+
+    return indentation or ''
 
 
 def strip_docstring(docstring):
@@ -310,8 +382,7 @@ def wrap_description(text, indentation, wrap_length):
     and bulleted lists alone.
 
     """
-    # Do not modify leading indentation unnecessarily.
-    text = text.lstrip('\n')
+    text = strip_leading_blank_lines(text)
 
     # Do not modify doctests at all.
     if '>>>' in text:
@@ -320,10 +391,13 @@ def wrap_description(text, indentation, wrap_length):
     text = reindent(text, indentation).rstrip()
 
     # Ignore possibly complicated cases.
-    if (wrap_length <= 0 or
-            re.search(r'\n\s*\n', text) or
-            re.search(r'[0-9]\.', text) or
-            re.search(r'[\-*:]', text)):
+    if (
+        wrap_length <= 0 or
+        re.search(r'\n\s*\n', text) or
+        re.search(r'[0-9]\.', text) or
+        re.search(r'[\-*:=]', text) or
+        is_some_sort_of_list(text)
+    ):
         return text
 
     return indentation + '\n'.join(
@@ -333,6 +407,19 @@ def wrap_description(text, indentation, wrap_length):
                       subsequent_indent=indentation)).strip()
 
 
+def strip_leading_blank_lines(text):
+    """Return text with leading blank lines removed."""
+    split = text.splitlines()
+
+    found = 0
+    for index, line in enumerate(split):
+        if line.strip():
+            found = index
+            break
+
+    return '\n'.join(split[found:])
+
+
 def open_with_encoding(filename, encoding, mode='r'):
     """Return opened file with a specific encoding."""
     return io.open(filename, mode=mode, encoding=encoding,
@@ -363,6 +450,7 @@ def format_file(filename, args, standard_out):
         formatted_source = format_code(
             source,
             summary_wrap_length=args.wrap_summaries,
+            description_wrap_length=args.wrap_descriptions,
             pre_summary_newline=args.pre_summary_newline,
             post_description_blank=args.post_description_blank)
 
