@@ -312,6 +312,26 @@ Hello.
                     if len(line.split()) > 1:
                         self.assertLessEqual(len(line), max_length)
 
+    def test_format_docstring_with_weird_indentation_and_punctuation(self):
+        self.assertEqual('''
+    """Creates and returns four was awakens to was created tracked ammonites
+    was the fifty, arithmetical four was pyrotechnic to pyrotechnic physicists.
+
+    `four' falsified x falsified ammonites
+    to awakens to. `created' to ancestor was four to x dynamo to was
+    four ancestor to physicists().
+
+    """
+'''.strip(),
+            docformatter.format_docstring('    ', '''
+    """Creates and returns four was awakens to was created tracked
+       ammonites was the fifty, arithmetical four was pyrotechnic to
+       pyrotechnic physicists. `four' falsified x falsified ammonites
+       to awakens to. `created' to ancestor was four to x dynamo to was
+       four ancestor to physicists().
+    """
+'''.strip(), summary_wrap_length=79))
+
     def test_format_code(self):
         self.assertEqual(
             '''\
@@ -690,7 +710,7 @@ def foo():
     def test_split_summary_and_description_complex(self):
         self.assertEqual(
             ('This is the first',
-             'This is the second. This is the third.'),
+             '\nThis is the second. This is the third.'),
             docformatter.split_summary_and_description(
                 'This is the first\n\nThis is the second. This is the third.'))
 
@@ -719,7 +739,7 @@ def foo():
                              'This is the first\nWashington'))
 
     def test_split_summary_and_description_with_list_on_other_line(self):
-        self.assertEqual(('Test\n    test', '@blah'),
+        self.assertEqual(('Test\n    test', '    @blah'),
                          docformatter.split_summary_and_description('''\
     Test
     test
@@ -786,6 +806,35 @@ Try this and this and this and this and this and this and this at
     email : string
 '''))
 
+    def test_split_summary_and_description_with_column_of_items(self):
+        text = """
+        one
+        two
+        three
+        four
+        five
+        six
+        seven
+        eight
+        nine
+        ten
+        eleven
+        """
+        self.assertEqual(
+            ('', text),
+            docformatter.split_summary_and_description(text))
+
+    def test_split_summary_and_description_with_abbreviation(self):
+        for text in  ['Test e.g. now'
+                      'Test i.e. now',
+                      'Test Dr. now',
+                      'Test Mr. now',
+                      'Test Mrs. now',
+                      'Test Ms. now']:
+            self.assertEqual(
+                (text, ''),
+                docformatter.split_summary_and_description(text))
+
     def test_normalize_summary(self):
         self.assertEqual(
             'This is a sentence.',
@@ -846,6 +895,27 @@ Try this and this and this and this and this and this and this at
             docformatter.reindent(description, indentation='    ')
         )
 
+    def test_find_shortest_indentation(self):
+        self.assertEqual(
+            ' ',
+            docformatter._find_shortest_indentation(['    ', ' b', '  a']))
+
+    def test_split_first_sentence(self):
+        self.assertEqual(
+            ('This is a sentence.', ' More stuff. And more stuff.   .!@#$%'),
+            docformatter.split_first_sentence(
+            'This is a sentence. More stuff. And more stuff.   .!@#$%'))
+
+        self.assertEqual(
+            ('This e.g. sentence.', ' More stuff. And more stuff.   .!@#$%'),
+            docformatter.split_first_sentence(
+            'This e.g. sentence. More stuff. And more stuff.   .!@#$%'))
+
+        self.assertEqual(
+            ('This is the first:', '\none\ntwo'),
+            docformatter.split_first_sentence(
+            'This is the first:\none\ntwo'))
+
 
 class TestSystem(unittest.TestCase):
 
